local SPBMJR, SPBVER = "SolacePartyBuffs", "4.0.1"

local SolaceLib = _G["SolaceLib"]

SolacePartyBuffs = CreateFrame("Frame")
SolacePartyBuffs:RegisterEvent("UNIT_AURA")
SolacePartyBuffs:RegisterEvent("VARIABLES_LOADED")
SolacePartyBuffs:RegisterEvent("PARTY_MEMBERS_CHANGED")
SolacePartyBuffs:RegisterEvent("PLAYER_ENTERING_WORLD")
SolacePartyBuffs:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
SolacePartyBuffs:RegisterEvent("PLAYER_TALENT_UPDATE")
SolacePartyBuffs.version = SPBVER

SolacePartyBuffs.Options = {
	["max_party_buffs"] = 32,
	["max_party_debuffs"] = 40,
	["filter_castable"] = true
}

SolacePartyBuffs.CanDispel = {
	PRIEST = {
		Magic = true,
		Disease = true
	},
	SHAMAN = {
		Curse = true,
		Magic = true
	},
	PALADIN = {
		Poison = true,
		Disease = true,
		Magic = true
	},
	MAGE = {
		Curse = true,
	},
	DRUID = {
		Curse = true,
		Poison = true,
		Magic = true
	}
}

SolacePartyBuffs.frameref = {
	["party1"] = "SolacePartyBuffFrame1",
	["party2"] = "SolacePartyBuffFrame2",
	["party3"] = "SolacePartyBuffFrame3",
	["party4"] = "SolacePartyBuffFrame4"
}

SolacePartyBuffs.blizzframes = {
	["party1"] = "PartyMemberFrame1",
	["party2"] = "PartyMemberFrame2",
	["party3"] = "PartyMemberFrame3",
	["party4"] = "PartyMemberFrame4"
}

function SolacePartyBuffs:ErrorMsg(name, index)
	DEFAULT_CHAT_FRAME:AddMessage("/solace"..SolaceLib.commands[index].cmd..": "..SolaceLib.commands[index].desc)
	DEFAULT_CHAT_FRAME:AddMessage(SPB_CURRENT..SolacePartyBuffs.Options[name])
end

function SolacePartyBuffs:buffshandler(msg, index)
	if msg then
		msg = tonumber(msg)
		if ( msg > 0 and msg < 33 ) then
			if ( msg < SolacePartyBuffs.Options.max_party_buffs ) then
				for j = 1, GetNumPartyMembers() do
					local frame = "SolacePartyBuffFrame"..j
					for i = msg, SolacePartyBuffs.Options.max_party_buffs do
						local button = _G[frame.."Buff"..i]
						if button then
							button:Hide()
						end
					end
				end
			end
			SolacePartyBuffs.Options.max_party_buffs = msg
			for i = 1, GetNumPartyMembers() do
				SolacePartyBuffs:UpdateBuffs("party"..i)
			end
			
			DEFAULT_CHAT_FRAME:AddMessage(SPB_PBUFFS_CHANGED..msg)
		else
			SolacePartyBuffs:ErrorMsg("max_party_debuffs", index)
		end
	else
		SolacePartyBuffs:ErrorMsg("max_party_buffs",index)
	end
end

function SolacePartyBuffs:debuffshandler(msg, index)
	if msg then
		msg = tonumber(msg)
		if ( msg > 0 and msg < 33 ) then
			if ( msg < SolacePartyBuffs.Options.max_party_debuffs ) then
				for j = 1, GetNumPartyMembers() do
					local frame = "SolacePartyDebuffFrame"..j
					for i = msg, SolacePartyBuffs.Options.max_party_debuffs do
						local button = _G[frame.."Debuff"..i]
						if button then
							button:Hide()
						end
					end
				end
			end
			SolacePartyBuffs.Options.max_party_debuffs = msg
			for i = 1, GetNumPartyMembers() do
				SolacePartyBuffs:UpdateBuffs("party"..i)
			end
			
			DEFAULT_CHAT_FRAME:AddMessage(SPB_PDEBUFFS_CHANGED..msg)
		else
			SolacePartyBuffs:ErrorMsg("max_party_debuffs", index)
		end
	else
		SolacePartyBuffs:ErrorMsg("max_party_debuffs",index)
	end
end

function SolacePartyBuffs:filterhandler(msg, index)
	if msg then
		if ( msg == "on" ) then
			SolacePartyBuffs.Options.filter_castable = true
			DEFAULT_CHAT_FRAME:AddMessage(SPB_PFILTER_CHANGED..SPB_ENABLED)
		elseif ( msg == "off" ) then
			SolacePartyBuffs.Options.filter_castable = false
			DEFAULT_CHAT_FRAME:AddMessage(SPB_PFILTER_CHANGED..SPB_DISABLED)
		else
			DEFAULT_CHAT_FRAME:AddMessage("/solace"..SolaceLib.commands[index].cmd..": "..SolaceLib.commands[index].desc)
			if ( SolacePartyBuffs.Options.filter_castable == true ) then
				DEFAULT_CHAT_FRAME:AddMessage(SPB_CURRENT..SPB_ENABLED)
			else
				DEFAULT_CHAT_FRAME:AddMessage(SPB_CURRENT..SPB_DISABLED)
			end
		end
		for i = 1, GetNumPartyMembers() do
			SolacePartyBuffs:UpdateBuffs("party"..i)
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("/solace"..SolaceLib.commands[index].cmd..": "..SolaceLib.commands[index].desc)
		if ( SolacePartyBuffs.Options.filter_castable == true ) then
			DEFAULT_CHAT_FRAME:AddMessage(SPB_CURRENT..SPB_ENABLED)
		else
			DEFAULT_CHAT_FRAME:AddMessage(SPB_CURRENT..SPB_DISABLED)
		end
	end
end

function SolacePartyBuffs:onEvent(event, ...)
	SolaceTargetFrame:onEvent(event, ...)
	local arg1 = ...
	if ( event == "VARIABLES_LOADED" ) then
		self["commandsInit"] = SolaceLib["commandsInit"]
		self:commandsInit("maxpartybuffs", SPB_MAXPARTYBUFFS, "buffshandler")
		self:commandsInit("maxpartydebuffs", SPB_MAXPARTYDEBUFFS, "debuffshandler")
		self:commandsInit("partyaurafilter", SPB_FILTERDEBUFFS, "filterhandler")
		DEFAULT_CHAT_FRAME:AddMessage(SPB_LOADED)
	end
	if ( event == "UNIT_AURA" ) then
		for k, v in pairs(SolacePartyBuffs.frameref) do
			if ( arg1 == k ) then
				SolacePartyBuffs:UpdateBuffs(arg1)
				return
			end
		end
	end
	if ( event == "PARTY_MEMBERS_CHANGED" or event == "PLAYER_ENTERING_WORLD" ) then
		for i = 1, GetNumPartyMembers() do
			self:UpdateBuffs("party"..i)
		end
	end
	if ( event == "PLAYER_TALENT_UPDATE" or event == "ACTIVE_TALENT_GROUP_CHANGED" or event == "PLAYER_ENTERING_WORLD" ) then
		local _, eClass = UnitClass("player")
		if ( self.CanDispel[eClass] ) then
			self:TalentDetect(eClass)
		end
	end
end

SolacePartyBuffs:SetScript("OnEvent", SolacePartyBuffs["onEvent"])

function SolacePartyBuffs:UpdateBuffs(unit)
--~ 	DEFAULT_CHAT_FRAME:AddMessage("UpdateBuffs called for unit "..unit)
--~ 	Hide the Blizzard debuffs.
	for i = 1, 4 do
		_G[self.blizzframes[unit].."Debuff"..i]:Hide()
	end
	
	local frame = self.frameref[unit]
	local button, buttonname
	local name, rank, icon, count, duration, expirationTime
	local buffCount
	local cooldown, startCooldownTime
	local filter_castable = nil
	
	if SolacePartyBuffs.Options.filter_castable then filter_castable = "PLAYER" end
	
	local i = 1
	while ( i <= SolacePartyBuffs.Options.max_party_buffs ) do
		name, rank, icon, count, debuffType, duration, expirationTime = UnitBuff(unit, i, filter_castable)
		buttonname = frame.."Buff"..i
		button = _G[buttonname]
		
		if ( not button ) then
			if ( not icon ) then
				break
			else
				button = CreateFrame("Button", buttonname, _G[frame], "SolacePartyBuffButtonTemplate")
			end
		end
		
		if ( icon ) then
			_G[buttonname.."Icon"]:SetTexture(icon)
			buffCount = _G[buttonname.."Count"]
			
			if (count > 1) then
				buffCount:SetText(count)
				buffCount:Show()
			else
				buffCount:Hide()
			end
			
			cooldown = _G[buttonname.."Cooldown"]
			if ( duration ) then
				if ( duration > 0 ) then
					cooldown:Show()
					CooldownFrame_SetTimer(cooldown, expirationTime - duration, duration, 1)
				else
					cooldown:Hide()
				end
			else
				cooldown:Hide()
			end
			
			button:SetID(i)
			
			button:ClearAllPoints()
			self:PositionButtons(frame.."Buff", i, frame, "Buff")
			
			button:Show()
		else
			button:Hide()
		end
		i = i + 1
	end
	
	local debuffType, color
	local debuffCount, debuffBorder
	local buttonname, button
	local debuffIdx = 0
	
	i = 1
	while ( i <= SolacePartyBuffs.Options.max_party_debuffs ) do
		
		name, rank, icon, count, debuffType, duration, expirationTime, i = SolacePartyBuffs:UnitDebuff(unit, i)
		
		debuffIdx = debuffIdx + 1
		buttonname = frame.."Debuff"..debuffIdx
		button = _G[buttonname]
		
		if ( not button ) then
			if ( not icon ) then
				break
			else
				button = CreateFrame("Button", buttonname, _G[frame], "SolacePartyDebuffButtonTemplate")
			end
		end
		
		if ( icon ) then
			debuffBorder = _G[buttonname.."Border"]
			
			_G[buttonname.."Icon"]:SetTexture(icon)
			debuffCount = _G[buttonname.."Count"]
			
			if ( debuffType ) then
				color = DebuffTypeColor[debuffType]
			else
				color = DebuffTypeColor["none"]
			end
			
			if (count > 1) then
				debuffCount:SetText(count)
				debuffCount:Show()
			else
				debuffCount:Hide()
			end
			
			cooldown = _G[buttonname.."Cooldown"]
			if ( duration ) then
				if ( duration > 0 ) then
					cooldown:Show()
					CooldownFrame_SetTimer(cooldown, expirationTime - duration, duration, 1)
				else
					cooldown:Hide()
				end
			else
				cooldown:Hide()
			end
			
			debuffBorder:SetVertexColor(color.r, color.g, color.b)
			
			button:SetID(debuffIdx)
			
			button:ClearAllPoints()
			self:PositionButtons(frame.."Debuff", debuffIdx, frame, "Debuff")
			
			button:Show()
		else
			button:Hide()
		end
		i = i + 1
	end
end

function SolacePartyBuffs:PositionButtons(name, index, frame, btype)
	local button = _G[name..index]
	local debuffBorder = _G[name..index.."Border"]
	if ( btype == "Buff" ) then
		if ( index == 1 ) then
			button:SetPoint("TOPLEFT", _G[frame], "TOPLEFT", 0, 0)
		elseif ( index == 17 ) then
			button:SetPoint("TOPLEFT", _G[name.."3"], "BOTTOM", 0, -2)
		else
			button:SetPoint("TOPLEFT", _G[name..(index - 1)], "TOPRIGHT", 2, 0)
		end
	else
		if ( index == 1 ) then
			debuffBorder:Hide()
			button:SetPoint("TOPLEFT", _G[frame], "TOPLEFT", 75, 38)
			debuffBorder:SetPoint("CENTER", button, "CENTER", 0, 0)
			debuffBorder:Show()
		else
			button:SetPoint("TOPLEFT", _G[name..(index - 1)], "TOPRIGHT", 2, 0)
		end
	end
end

function SolacePartyBuffs:UnitDebuff(unit,id)
if not unit then return end
	local name, rank, icon, count, debuffType, duration, timeLeft = UnitDebuff(unit,id)

	if self.Options.filter_castable then
		local _, eClass = UnitClass("player")
		while ( ( id <= self.Options.max_party_debuffs ) and not ( ( self.CanDispel[eClass] and ( self.CanDispel[eClass][debuffType] == true ) ) or ( eClass == "PRIEST" and aura == "Interface\\Icons\\Spell_Holy_AshesToAshes" ) ) ) do
			id=id+1
			name, rank, icon, count, debuffType, duration, timeLeft = UnitDebuff(unit,id)
		end
		return name, rank, icon, count or 0, debuffType, duration, timeLeft, id
	else
		return name, rank, icon, count or 0, debuffType, duration, timeLeft, id
	end
end

function SolacePartyBuffs:TalentDetect(eClass)
	local findTalents = function(tab, searchval)
		for i = 1, GetNumTalents(tab, false, false) do
			local name, _, _, _, rank, _ = GetTalentInfo(tab, i, false, false, nil)
			if ( rank > 0 and name == searchval ) then
				return true
			end
		end
		return false
	end
	if ( eClass == "DRUID" ) then
		if ( findTalents(3, SPB_NATURES_CURE) ) then
			SolacePartyBuffs.CanDispel[eClass]["Magic"] = true
		else
			SolacePartyBuffs.CanDispel[eClass]["Magic"] = false
		end
	elseif ( eClass == "SHAMAN" ) then
		if ( findTalents(3, SPB_IMP_CLEANSE_SPI) ) then
			SolacePartyBuffs.CanDispel[eClass]["Magic"] = true
		else
			SolacePartyBuffs.CanDispel[eClass]["Magic"] = false
		end
	elseif ( eClass == "PALADIN" ) then
		if ( findTalents(1, SPB_SACRED_CLEANSING) ) then
			SolacePartyBuffs.CanDispel[eClass]["Magic"] = true
		else
			SolacePartyBuffs.CanDispel[eClass]["Magic"] = false
		end
	end
end

local oldPartyMemberBuffTooltip_Update = PartyMemberBuffTooltip_Update
local function newPartyMemberBuffTooltip_Update(self)
	oldPartyMemberBuffTooltip_Update(self)    
	if (not string.find(self.unit,"pet")) then
		PartyMemberBuffTooltip:Hide()
	end
end
PartyMemberBuffTooltip_Update = newPartyMemberBuffTooltip_Update

